<p align="center">
    <a href="https://sylius.com" target="_blank">
        <img src="https://demo.sylius.com/assets/shop/img/logo.png" />
    </a>
</p>

<h1 align="center">Dedi Import Export Plugin</h1>

## Description

Le plugin se base sur <a href="https://github.com/FriendsOfSylius/SyliusImportExportPlugin" target="_blank">FosSyliusImportExportPlugin</a>, qui permet l'import/export dans les grids du back-office Sylius, et expose en plus une api pour des exports "custom".

Le plugin `FosSyliusImportExportPlugin` étant directement lié aux resources et grid sylius, ce plugin permet l'ajout d'exports découplé des resources et des grid.

## Installation

1. Executer `composer require dedi/sylius-import-export`.

2. Enregistrer les plugins dans bundles.php :

    ```php
    // config/bundles.php
    
    return [
        //...
        FriendsOfSylius\SyliusImportExportPlugin\FOSSyliusImportExportPlugin::class => ['all' => true],
        Dedi\SyliusImportExportPlugin\DediSyliusImportExportPlugin::class => ['all' => true],
    ];
    ```
## Configuration

### FOSSyliusImportExportPlugin

```yaml
# config/packages/fos_sylius_import_export.yaml

fos_sylius_import_export:
    importer:
        # set to false to not add an upload form to the entity overview pages
        web_ui:               true
        # set to an integer value bigger than 0 to flush the object manager in regular intervals
        batch_size:           20
        # if incomplete rows (ie. missing required fields) should be considered failures
        fail_on_incomplete:   false
        # if to stop the import process in case of a failure
        stop_on_failure:      false
    exporter:
      # set to false to not add export buttons
        web_ui:               true   
```

```yaml
# config/packages/dedi_sylius_import_export.yaml
imports:
  - { resource: "@DediSyliusImportExportPlugin/Resources/config/app/config.yaml" }
```

Le plugin est utilisé et pré-configuré pour générer les exports CSV par défaut de certaines grid :

- Produit
- Client
- Commande
- Promotion
- Paiement

Les services permettant ces exports sont décrits dans `src/Resources/config/fos_sylius_import_export_services.yaml`

### Tags

```yaml
# config/services.yaml

_instanceof:
    Dedi\SyliusImportExportPlugin\Exporter\GridAwareExporterInterface:
        tags: [ 'dedi.exporter.grid_exporter' ]
    Dedi\SyliusImportExportPlugin\Exporter\ExporterInterface:
        tags: [ 'dedi.exporter.exporter' ]
```


### Routing

Importer les routes dans votre projet :

```yaml
# config/routes/dedi_sylius_import_export.yaml

dedi_sylius_import_export:
  resource: "@DediSyliusImportExportPlugin/Resources/config/routing.yml"
```

La même route est utilisé pour tous les exports en back-office et prend en seul paramètre l'`identifier` renseigné dans l'exporter.

```yaml
app_admin_custom_export:
  controller: Dedi\SyliusImportExportPlugin\Controller\ExportController
  path: /admin/custom-export/{identifier}
  methods: ['GET']
```



## Utilisation

### DataProvider

    
Un dataprovider renvoi les données qui seront utilisés dans l'export.
Celui-ci doit implémenter l'interface  `Dedi\SyliusImportExportPlugin\DataProvider\DataProviderInterface`
dont la méthode `getData` renvoie une `Doctrine\Common\Collections\Collection`

L'utilisation de collection permettra à l'avenir d'envoyer une collection de DTO par exemple et de les serializer (non supporté pour le moment); 

```php
<?php

namespace App\DataProvider;


use Dedi\SyliusImportExportPlugin\DataProvider\DataProviderInterface;
use Doctrine\Common\Collections\ArrayCollection;use Doctrine\Common\Collections\Collection;
use Sylius\Bundle\ProductBundle\Doctrine\ORM\ProductRepository;

class FooDataProvider implements DataProviderInterface
{
    public function getData(): Collection
    {
        $collection = new ArrayCollection();
        $collection->add(['foo' => 'bar', 'foo2' => 'bar2']);

        return $collection;
    }
}
```

### Exporter
    
L'exporter par défaut `Dedi\SyliusImportExportPlugin\Exporter\DefaultExporter` est disponible pour un minimum de configuration.
Celui-ci prend en argument :
- Une `DataProviderInterface`
- Un `FormaterInterface`
- Un identifiant unique
- Un libellé de fichier de sortie
- Un libellé (affiché en back-office)
- Optionnel : Code du grid sylius 

```yaml
# config/services.yaml

app_foo_exporter:
    class: Dedi\SyliusImportExportPlugin\Exporter\DefaultExporter
    arguments:
        - '@App\Exporter\FooDataProvider'
        - '@Dedi\SyliusImportExportPlugin\Formater\CsvFormater' // formater
        - 'foo_exporter' // identifiant unique
        - 'foo-export' // nom du fichier de sortie
        - 'app.exporters.foo.label' // clef de trad
        - 'sylius_admin_product' // code du grid sylius (optionnel)
```

Il est possible de créer son propre exporter, et d'implémenter au choix 2 interfaces.
- `Dedi\SyliusImportExportPlugin\Exporter\ExporterInterface`
  
  Interface de base pour que l'export fonctionne. 
- `Dedi\SyliusImportExportPlugin\Exporter\GridAwareExporterInterface`
  
  Etend l'interface précédente, avec une méthode pour préciser le code de la grid.
  Seuls les exporter implémentant cette interface sont susceptibles d'être affichés dans le dropdown des exports en back-office.  
  Si l'exporter ne s'affiche pas dans le dropdown du grid défini dans la configuration, vérifier qu'un event correspondant au grid est bien défini dans la configuration du `ExportButtonGridListener` (src/Resources/config/services.yml:26).

  Les filtres des grid sont automatiquement passés dans le paramètre `criteria` de la query string, il est donc possible de les utiliser pour filtrer son export en utiliser la `RequestStack` dans son `DataProvider`

  ```php
    <?php
    
    namespace App\Exporter;
    
    
    use Dedi\SyliusImportExportPlugin\DataProvider\DataProviderInterface;
    use Dedi\SyliusImportExportPlugin\Exporter\GridAwareExporterInterface;
    use Dedi\SyliusImportExportPlugin\Formater\FormaterInterface;
    
    class FooExporter implements GridAwareExporterInterface
    {
        protected DataProviderInterface $dataProvider;
    
        protected FormaterInterface $formater;
  
        public function __construct(
            DataProviderInterface $dataProvider,
            FormaterInterface $writer
        ) {
            $this->dataProvider = $dataProvider;
            $this->formater = $writer;
        }
  
        public function getDataProvider(): DataProviderInterface
        {
            return $this->dataProvider;
        }
  
        public function getFormater(): FormaterInterface
        {
            return $this->formater;
        }
  
        public function getIdentifier(): string
        {
            return 'foo-export';
        }
    
        public function getFilename(): string
        {
            return 'foo_export';
        }
    
        public function getLabel(): string
        {
            return 'app.exporters.foo.label';
        }
    
        public function getGridCode(): string
        {
            return 'sylius_admin_product';
        }
    }
  ````


### Formater

Le formater permet la transformation d'une `Doctrine\Common\Collections\Collection` en format de sortie souhaité.

Formats supportés :

- CSV : `Dedi\SyliusImportExportPlugin\Formater\CsvFormater`


Il est possible d'ajouter des formater en implémentant l'interface `Dedi\SyliusImportExportPlugin\Formater\FormaterInterface`


### Processor

Le processor `Dedi\SyliusImportExportPlugin\Processor\ExportProcessor` génère le fichier depuis un `Dedi\SyliusImportExportPlugin\Exporter\ExporterInterface`.
Celui-ci est utilisé par le controller par défaut utilisé pour les exports en back-office.

L'interface `Dedi\SyliusImportExportPlugin\Processor\ExportProcessorInterface` est disponible pour céer son propre processor selon les besoins.